/*
 * Copyright (C) 2021      Cray UK
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "mmb_error.h"
#include "mmb_memory.h"

/**
 * NOTE:
 * `mmb_hwloc_topology_h` is typedef'd in mmb_memory.h as `mmb_hwloc_topology_h`
 * or as `void` depending on whether ENABLE_DISCOVERY is true or not.
 */

/**
 * @brief      Returns the hwloc topology used to discover the machine topology
 *             if Mamba was configure with the automatic discovery enabled.
 *
 * It is erroneous to call this function if the library was not built with
 * discovery enabled.  Users have to check with \p mmb_discovery_is_enabled()
 * first.
 *
 * @param      topo     Pointer to return the topology structure.
 *
 * @return     \c MMB_OK on success.
 */
mmbError mmb_get_hwloc_topology(mmb_hwloc_topology_t **topo);

/**
 * @brief       Query whether the library was built with automatic topology
 *              discovery.
 *
 * If the library was built with automatic topology discovery, the topology is
 * based on the informations provided by hwloc.  Without the automatic
 * discovery, the user has to register manually the different memory spaces
 * their program requires.
 *
 * @param       is_enabled  pointer to boolean.  Cannot be NULL.
 *
 * @return      \c MMB_OK on success.
 */
mmbError mmb_discovery_is_enabled(int *is_enabled);

