/*
 * Copyright (C) 2018-2020 Cray UK
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MEMORY_CONFIG_H
#define MEMORY_CONFIG_H

#ifndef MMB_MAX_SPACES
#  define MMB_MAX_SPACES 5
#endif /* MMB_MAX_SPACES */

/*
 * Memory provider configuration
 */
#ifndef MMB_CONFIG_PROVIDER_DEFAULT_ENV_NAME
#  define MMB_CONFIG_PROVIDER_DEFAULT_ENV_NAME "MMB_CONFIG_PROVIDER_DEFAULT"
#endif /* MMB_CONFIG_PROVIDER_DEFAULT_ENV_NAME */

#ifndef MMB_CONFIG_PROVIDER_DEFAULT
#  define MMB_CONFIG_PROVIDER_DEFAULT MMB_SYSTEM
#endif /* MMB_CONFIG_PROVIDER_DEFAULT */

/*
 * Strategy configuration
 */
#ifndef MMB_CONFIG_STRATEGY_DEFAULT_ENV_NAME
#  define MMB_CONFIG_STRATEGY_DEFAULT_ENV_NAME "MMB_CONFIG_STRATEGY_DEFAULT"
#endif /* MMB_CONFIG_STRATEGY_DEFAULT_ENV_NAME */

#ifndef MMB_CONFIG_STRATEGY_DEFAULT
#  define MMB_CONFIG_STRATEGY_DEFAULT MMB_STRATEGY_NONE
#endif /* MMB_CONFIG_STRATEGY_DEFAULT */

/*
 * Memory interface's default name
 */
#ifndef MMB_CONFIG_INTERFACE_NAME_DEFAULT_ENV_NAME
#  define MMB_CONFIG_INTERFACE_NAME_DEFAULT_ENV_NAME "MMB_CONFIG_INTERFACE_NAME_DEFAULT"
#endif /* MMB_CONFIG_INTERFACE_NAME_DEFAULT_ENV_NAME */

#ifndef MMB_CONFIG_INTERFACE_NAME_DEFAULT
#  define MMB_CONFIG_INTERFACE_NAME_DEFAULT "default"
#endif /* MMB_CONFIG_INTERFACE_NAME_DEFAULT */

/*
 * GPU execution context configuration
 */
#ifndef MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT_ENV_NAME
#  define MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT_ENV_NAME "MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT"
#endif /* MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT_ENV_NAME */

#ifndef MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT
#  if HAVE_CUDA
#    define MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT MMB_GPU_CUDA
#  elif HAVE_OPENCL
#    define MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT MMB_GPU_OPENCL
#  else
     /* No GPU support at all */
#    define MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT MMB_EXECUTION_CONTEXT_NONE
#  endif
#else
  /* Check for misconfiguration at compile time */
#  if MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT != MMB_GPU_CUDA  \
      && MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT != MMB_GPU_OPENCL
#    error Invalid GPU context configured.
#  elif MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT == MMB_GPU_OPENCL
     /* To be removed when OpenCL support will be added */
#    warning OpenCL is not yet supported. The library is likely to fail at run-time.
#  endif
#endif /* MMB_CONFIG_EXECUTION_CONTEXT_GPU_DEFAULT */

#endif /* MEMORY_CONFIG_H */
