#include "maestro/i_memory.h"

#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

void *
mstro_alloc_lockable(size_t size, bool lock_immediately)
{
  void *ptr=NULL;
  int retval;
  /* 
   * alloc memory aligned to a page, to prevent two mlock() in the
   * same page.
   */
  retval = posix_memalign(&ptr, (size_t) sysconf(_SC_PAGESIZE), size);
  
  /* return NULL on failure */
  if (retval)
    return NULL;
  
  /* lock this buffer into RAM */
  if(lock_immediately)
    if (mlock(ptr, size)) {
      free(ptr);
      return NULL;
    }
  return ptr;
}

void
mstro_unlock_lockable(void *ptr, size_t size)
{
    munlock(ptr, size);
}

mstro_status 
mstro_free_lockable(void *ptr, size_t size, bool needs_unlock)
{
  if(ptr==NULL)
    return MSTRO_INVARG;
  
  if(needs_unlock)
    /* unlock the address range */
    mstro_unlock_lockable(ptr, size);

  /* free the memory */
  free(ptr);
  return MSTRO_OK;
}



