#include <stdio.h>
#include <assert.h>
#include "maestro.h"

int main(int argc, char* argv[])
{
    assert(MSTRO_OK == mstro_init(getenv("MSTRO_WORKFLOW_NAME"), getenv("MSTRO_COMPONENT_NAME"), 0));

    mstro_cdo_selector selector = NULL;
    assert(MSTRO_OK == mstro_cdo_selector_create(NULL, NULL, "(has .maestro.core.cdo.name)", &selector));

    mstro_subscription offer_subscription;
    assert(MSTRO_OK == mstro_subscribe(selector, MSTRO_POOL_EVENT_OFFER,
        MSTRO_SUBSCRIPTION_OPTS_SIGNAL_BEFORE|MSTRO_SUBSCRIPTION_OPTS_REQUIRE_ACK,
        &offer_subscription));
    assert(MSTRO_OK == mstro_cdo_selector_dispose(selector));

    mstro_pool_event event;
    assert(MSTRO_OK == mstro_subscription_wait(offer_subscription, &event));
    const char* name = event->offer.cdo_name;
    mstro_cdo cdo;
    printf("Offer occured for: %s\n", event->offer.cdo_name);

    assert(MSTRO_OK == mstro_cdo_declare(name, MSTRO_ATTR_DEFAULT, &cdo));
    assert(MSTRO_OK == mstro_cdo_require(cdo));
    assert(MSTRO_OK == mstro_subscription_ack(offer_subscription, event));

    printf("Demanding and disposing alternate handle: %s", name);
    mstro_cdo another_handle;
    assert(MSTRO_OK == mstro_cdo_declare(name, MSTRO_ATTR_DEFAULT, &another_handle));
    assert(MSTRO_OK == mstro_cdo_require(another_handle));
    assert(MSTRO_OK == mstro_cdo_demand(another_handle));
    assert(MSTRO_OK == mstro_cdo_dispose(another_handle));

    assert(MSTRO_OK == mstro_cdo_retract(cdo));
    assert(MSTRO_OK == mstro_cdo_dispose(cdo));

    assert(MSTRO_OK == mstro_finalize());

    
    return 0;
}
